#!/usr/bin/env python

"""
HTML serialiser.

Copyright (C) 2017, 2018 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from moinformat.serialisers.common import escape_attr, escape_text, Serialiser

class HTMLSerialiser(Serialiser):

    "Serialisation of the page."

    def start_region(self, level, indent, type):
        l = []
        out = l.append
        if level:
            out("level-%d" % level)

        if indent:
            out("indent-%d" % indent)

        # NOTE: Encode type details for CSS.

        if type:
            out("type-%s" % escape_attr(type))

        self.out("<span class='%s'>" % " ".join(l))

    def end_region(self, level, indent, type):
        self.out("</span>")

    def start_block(self):
        self.out("<p>")

    def end_block(self):
        self.out("</p>")

    def start_defitem(self, pad, extra):
        self.out("<dd>")

    def end_defitem(self, pad, extra):
        self.out("</dd>")

    def start_defterm(self, pad):
        self.out("<dt>")

    def end_defterm(self, pad):
        self.out("</dt>")

    def start_emphasis(self):
        self.out("<em>")

    def end_emphasis(self):
        self.out("</em>")

    def start_heading(self, level, extra, pad):
        self.out("<h%d>" % level)

    def end_heading(self, level, pad, extra):
        self.out("</h%d>" % level)

    def start_larger(self):
        self.out("<big>")

    def end_larger(self):
        self.out("</big>")

    def start_link(self, target):
        self.out('<a href="%s">' % escape_attr(target))

    def end_link(self):
        self.out("</a>")

    def start_linktext(self):
        pass

    def end_linktext(self):
        pass

    list_tags = {
        "i" : "lower-roman",
        "I" : "upper-roman",
        "a" : "lower-latin",
        "A" : "upper-latin",
        }

    def _get_list_tag(self, marker):
        if marker:
            if marker[0].isdigit():
                return "ol", "decimal"
            style_type = self.list_tags.get(marker[0])
            if style_type:
                return "ol", style_type

        return "ul", None

    def start_list(self, indent, marker, num):
        tag, style_type = self._get_list_tag(marker)
        style = style_type and ' style="list-style-type: %s"' % escape_attr(style_type) or ""
        start = style_type and num is not None and ' start="%s"' % escape_attr(num) or ""
        self.out("<%s%s%s>" % (tag, style, start))

    def end_list(self, indent, marker, num):
        tag, style = self._get_list_tag(marker)
        self.out("</%s>" % tag)

    def start_listitem(self, indent, marker, space, num):
        self.out("<li>")

    def end_listitem(self, indent, marker, space, num):
        self.out("</li>")

    def start_monospace(self):
        self.out("<tt>")

    def end_monospace(self):
        self.out("</tt>")

    def start_smaller(self):
        self.out("<small>")

    def end_smaller(self):
        self.out("</small>")

    def start_strikethrough(self):
        self.out("<del>")

    def end_strikethrough(self):
        self.out("</del>")

    def start_strong(self):
        self.out("<strong>")

    def end_strong(self):
        self.out("</strong>")

    def start_subscript(self):
        self.out("<sub>")

    def end_subscript(self):
        self.out("</sub>")

    def start_superscript(self):
        self.out("<sup>")

    def end_superscript(self):
        self.out("</sup>")

    def start_table(self):
        self.out("<table>")

    def end_table(self):
        self.out("</table>")

    def start_table_attrs(self):
        pass

    def end_table_attrs(self):
        pass

    def start_table_cell(self, attrs):
        self.out("<td")
        if attrs and not attrs.empty():
            attrs.to_string(self)
        self.out(">")

    def end_table_cell(self):
        self.out("</td>")

    def start_table_row(self):
        self.out("<tr>")

    def end_table_row(self, trailing):
        self.out("</tr>")

    def start_underline(self):
        self.out("<span style='text-decoration: underline'>")

    def end_underline(self):
        self.out("</span>")

    def break_(self):
        pass

    def rule(self, length):
        self.out("<hr style='height: %dpt' />" % min(length, 10))

    def table_attr(self, name, value, concise, quote):
        self.out(" %s%s" % (escape_text(name), value is not None and
            "='%s'" % escape_attr(value) or ""))

    def text(self, s):
        self.out(escape_text(s))

serialiser = HTMLSerialiser

# vim: tabstop=4 expandtab shiftwidth=4
